local addonName, addonNS = ...

-- Initialise XrystalUI Namespace as needed
addonNS = addonNS or {}

addonNS.mountList = {}
addonNS.nextMount = -1

-- Create the button we will use
local DFMB = CreateFrame("CheckButton","XMount_DynamicFlightButton",UIParent,"XMountDFButton")
DFMB.Icon:SetTexture("Interface\\PetBattles\\MountJournalEmptyIcon")

-- This function as you can see will build a list of mounts available to the player
-- It is called when you log in and whenever a new mount is learnt
local function BuildDynamicFlightMountsList()
    
    --local mountIDs = C_MountJournal.GetMountIDs()
    local mountIDs = C_MountJournal.GetCollectedDragonridingMounts()
    local englishFaction, localizedFaction = UnitFactionGroup("player")
    addonNS.mountList = {}
    
    for i,v in ipairs(mountIDs) do
        local name, spellID, icon, isActive, isUsable, sourceType, isFavorite, isFactionSpecific, faction, shouldHideOnChar, isCollected, mountID, isForDragonriding = C_MountJournal.GetMountInfoByID(v)

        local isWrongFaction = isFactionSpecific and ( faction ~= englishFaction)
        
        if isCollected and (not isWrongFaction) and isUsable then
        
            table.insert(addonNS.mountList,
            {
                ["index"] = i,
                ["mountID"] = mountID,
                ["name"] = name,
                ["spellID"] = spellID,
                ["icon"] = icon,
                ["isActive"] = isActive,
                ["isUsable"] = isUsable,
                ["isCollected"] = isCollected,
                ["isForDragonriding"] = isForDragonriding,
            })
        end
    end
    
end

-- If we don't have a mount, then, why select a random one
-- It picks a random number, 
-- then modulates it by the number of mounts available to return a value starting from 0 and the number of mounts
-- It then uses that value plus 1 as the new mount index
-- Then sets the buttons texture and relevant identifiers
local function SelectNextMount()
    if #addonNS.mountList == 0 then return end
    local low,high = 1,500
    local rand = math.random(low,high)
    local fm = math.fmod(rand,#addonNS.mountList)    
    addonNS.nextMount = fm + 1    
    local icon = addonNS.mountList[addonNS.nextMount].icon
    local spell = addonNS.mountList[addonNS.nextMount].spellID
    DFMB.Icon:SetTexture(icon)
    DFMB.spellID = spell
    DFMB.mountID = addonNS.nextMount
end

-- The Mount and Dismount functionality
-- This should never be called when it shouldn't as we have 2 checks in place to stop this from happening
local function MountOrDismount(mount)
    if mount then 
        C_MountJournal.SummonByID(addonNS.mountList[addonNS.nextMount].mountID)
    else
        Dismount()
    end    
end


-- Validate whether the button is shown or not based on situation.  It can also be forced such as when combat starts and ends
local function ValidateButtonVisibility(forceVisibility)
    
    local show = false
    
    -- Generally used during combat situations to make sure the button isn't available when it shouldn't be
    if forceVisibility ~= nil then
        DFMB:SetShown(forceVisibility)
        return forceVisibility
    end
        
    -- If we don't have any dragonflight mounts then don't show the button
    if #addonNS.mountList > 0 and
        
    -- If we can't use Dynamic Flight, don't show the button
    IsAdvancedFlyableArea() and
            
    -- If we are indoors then, don't show the button
    not IsIndoors() then
        show = true
            
    end

    -- Whatever the result is, show it or not
    DFMB:SetShown(show)
    return show
    
end



-- Sets up our button
local function SetupButton()

    -- Make sure initial status of button is valid based on mounted status
    DFMB.isChecked = IsMounted()
    DFMB:SetChecked(DFMB.isChecked)

    -- If the button has been hidden then don't continue
    if not ValidateButtonVisibility() then return end
    
    -- Get the binding key ( if any ) for this button and display it
    local key = GetBindingKey("CLICK "..DFMB:GetName()..":LeftButton")
	local text = GetBindingText(key, 1)
    if ( text == "" ) then
        DFMB.HotKey:Hide()
    else
        DFMB.HotKey:SetText(text)
        DFMB.HotKey:Show()
    end
    
    
    -- Manage clicking of button 
    DFMB:SetScript("OnClick", function(self,button,...)
                        
        local update = false
            
        -- Check if we need to dismount and reshuffle or mount up
        if self.isChecked then 
            MountOrDismount(false)
            SelectNextMount()
            update = true
        else
            -- Just in case the button stays visible, this makes sure you don't try to mount in combat
            if not InCombatLockdown() then
                MountOrDismount(true)            
                update = true
            end
        end
            
        -- Update checked status
        if update then
            DFMB.isChecked = not DFMB.isChecked
            DFMB:SetChecked(DFMB.isChecked)
        end
            
    end)
    
    -- Handle GameTooltip Display
    DFMB:SetScript("OnEnter", function(self,motion)
        GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT")
        GameTooltip:SetMountBySpellID(self.spellID)
    end)
    
    DFMB:SetScript("OnLeave", function(self,motion)
        GameTooltip:Hide()
    end)
    
end

local eventWatcher = CreateFrame("Frame")
eventWatcher:RegisterEvent("ADDON_LOADED")
eventWatcher:SetScript("OnEvent",
function(self,event,...)
    if event == "ADDON_LOADED" then
        local arg1 = ...
        if arg1 == addonName then
            self:RegisterEvent("PLAYER_LOGIN")
            self:RegisterEvent("NEW_MOUNT_ADDED")
            self:RegisterEvent("PLAYER_REGEN_ENABLED")
            self:RegisterEvent("PLAYER_REGEN_DISABLED")
            self:RegisterEvent("ADDON_ACTION_BLOCKED")
            self:RegisterEvent("ZONE_CHANGED_INDOORS")
            self:RegisterEvent("ZONE_CHANGED")
            self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
        end

    elseif event == "PLAYER_LOGIN" then
        BuildDynamicFlightMountsList()
        SelectNextMount()
        SetupButton()
            
    elseif event == "NEW_MOUNT_ADDED" then
        BuildDynamicFlightMountsList()
        SelectNextMount()
        SetupButton()
            
    elseif event == "PLAYER_REGEN_ENABLED" then
        ValidateButtonVisibility(true)
        SetupButton()
           
    elseif event == "PLAYER_REGEN_DISABLED" then
        ValidateButtonVisibility(false)
        Dismount()
                
    elseif event == "ZONE_CHANGED_INDOORS" then
        Dismount()
        SetupButton()

    elseif event == "ZONE_CHANGED" then
        SetupButton()
            
    elseif event == "ZONE_CHANGED_NEW_AREA" then
        SetupButton()
            
    end
        
end)
